/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_TLV320AIC31XX_H
#define __DT_TLV320AIC31XX_H

#define MICBIAS_2_0V		1
#define MICBIAS_2_5V		2
#define MICBIAS_AVDDV		3

#define PLL_CLKIN_MCLK		0x00
#define PLL_CLKIN_BCLK		0x01
#define PLL_CLKIN_GPIO1		0x02
#define PLL_CLKIN_DIN		0x03

#endif /* __DT_TLV320AIC31XX_H */
