/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_SUN55I_A523_PPU_H_
#define _DT_BINDINGS_POWER_SUN55I_A523_PPU_H_

#define PD_DSP			0
#define PD_NPU			1
#define PD_AUDIO		2
#define PD_SRAM			3
#define PD_RISCV		4

#endif /* _DT_BINDINGS_POWER_SUN55I_A523_PPU_H_ */
