/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Common values for the GHASH hash function
 */

#ifndef __CRYPTO_GHASH_H__
#define __CRYPTO_GHASH_H__

#define GHASH_BLOCK_SIZE	16
#define GHASH_DIGEST_SIZE	16

#endif
