/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2013 Freescale Semiconductor, Inc.
 */

#ifndef __LINUX_CLK_MXS_H
#define __LINUX_CLK_MXS_H

int mxs_saif_clkmux_select(unsigned int clkmux);

#endif
